﻿/*
@@@BUILDINFO@@@ PaperTexture.jsx 1.2.0  19-Apr-2012
*/
﻿
/**************************************************************************
	ADOBE SYSTEMS INCORPORATED
	 Copyright 2012 Adobe Systems Incorporated
	 All Rights Reserved.

	NOTICE:  Adobe permits you to use, modify, and distribute this file
	in accordance with the terms of the Adobe license agreement accompanying
	it.  If you have received this file from a source other than Adobe, then
	your use, modification, or distribution of it requires the prior written
	permission of Adobe.
**************************************************************************/

// i didn't indent cuz that would be annoying
try {
	

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// // Some globals at the top for easy editing
var paperTexture = new Object;
paperTexture.showAlerts = true; // use this for user alerts
paperTexture.debugAlerts = false; // use this for debug purposes only, should be false
paperTexture.leaveWatermarkOpenWhenProcessingOpenImages = false;
paperTexture.tempDocName = localize("$$$/papertexture/string/tempdocname=Paper Texture");
paperTexture.extraDebug = false;
paperTexture.defaultFolder = app.path + "/Plug-ins/Panels/Paper Texture/Textures";
paperTexture.keepTextureDocOpen = false;

///////////////////////////////////////////////////////////////////////////////
// Function: runProcess
// Usage: Main processing routine called from the Watermark panel
// Input: string of the object from the panel, must use eval on the parameters
// Return: <nothing>
///////////////////////////////////////////////////////////////////////////////
paperTexture.runProcess = function( inParamsAsSource ) {

	var oldPref = app.preferences.rulerUnits;

	var oldColorSettings = paperTexture.getColorSettings();
	
	paperTexture.setColorSettings( { askMismatchOpening:false, askMismatchPasting:false, askMissing:false } );
		
	try {

	    app.preferences.rulerUnits = Units.PIXELS;
	    
		var params = paperTexture.paramsToParams( inParamsAsSource, 0 /* version */ );
		
		if ( app.documents.length == 0 ) {
			var f = app.openDialog();
			if (f.length > 0) {
				app.open(File(f[0].fsName.toString()));
			} else {
				return;
			}
		}

		var targetDoc = activeDocument;
		
		if ( activeDocument.colorProfileType != ColorProfile.NONE ) {
			var targetProfile = activeDocument.colorProfileName;
		} else {
			var targetProfile = ColorProfile.NONE;
		}
			
		for ( var j = 0; j < params.filesArray.length; j++ ) {
			// strip the extension off
			var fileNameNoExtension = unescape( File( params.filesArray[j] ).name );
			fileNameNoExtension = fileNameNoExtension.split( "." );
			if ( fileNameNoExtension.length > 1 ) {
				fileNameNoExtension.length--;
			}
			fileNameNoExtension = fileNameNoExtension.join(".");
			var layerName = paperTexture.tempDocName + " " + fileNameNoExtension;
			for ( var i = 0; i < targetDoc.artLayers.length; i++ ) {
				if ( targetDoc.artLayers[i].name == layerName ) {
					targetDoc.artLayers[i].remove();	
				}
				if ( params.runRandomizer && targetDoc.artLayers[i].name.search(paperTexture.tempDocName) != -1 ) {
					targetDoc.artLayers[i].remove();	
				}	
			}
		}
		
		for ( var i = 0; i < params.filesArray.length; i++ ) {
	
			if (params.stateArray[i] == false) {
				continue;
			}

			var alreadyOpenFile = undefined;
			for ( var s = 0; s < app.documents.length; s++ ) {
				try {
					if ( params.filesArray[i].toString() == app.documents[s].fullName.toString() ) {
						alreadyOpenFile = app.documents[s];
						break;
					}
				} catch(e) { ; } // just go to the next file
			}
			if ( typeof(alreadyOpenFile) != "undefined" && alreadyOpenFile instanceof Document ) {
				textureDoc = alreadyOpenFile;
			} else {
				textureDoc = app.open( File ( params.filesArray[i] ) );
			}
			
			// strip the extension off
			var fileNameNoExtension = unescape( File( params.filesArray[i] ).name );
			fileNameNoExtension = fileNameNoExtension.split( "." );
			if ( fileNameNoExtension.length > 1 ) {
				fileNameNoExtension.length--;
			}
			fileNameNoExtension = fileNameNoExtension.join(".");
			
			var layerName = paperTexture.tempDocName + " " + fileNameNoExtension;
			
			var docOpened = textureDoc;
			
			var fullPath = params.filesArray[i];

			paperTexture.convertModeIfNeeded( textureDoc, undefined ); // pick the default mode if needed
 
 			textureDoc.resizeImage( targetDoc.width, targetDoc.height, targetDoc.resolution, ResampleMethod.BICUBICSHARPER ); // ResampleMethod.BICUBIC );
 			
 			if ( textureDoc.colorProfileType != targetDoc.colorProfileType ) {
 				if ( targetProfile != ColorProfile.NONE ) {
					paperTexture.convertToProfile( targetProfile );
				} else {
					activeDocument.colorProfileType = ColorProfile.NONE;
				}
			}
 			
 			paperTexture.duplicateLayerToDocument( layerName, targetDoc.name );
 			
 			if ( ! paperTexture.keepTextureDocOpen )
 				textureDoc.close( SaveOptions.DONOTSAVECHANGES );

			activeDocument = targetDoc;
			
			targetDoc.activeLayer.blendMode = paperTexture.textToBlendMode( params.ddBlendMode );
			
			paperTexture.addMaskAsNeeded( true /* reveals */);
			
			paperTexture.fitOnScreen();
			
			processedOneFile = true;
			
		} // end for loop
		
	} // main try block

	catch(e) { 
		if (paperTexture.showAlerts) alert(e +" : " + e.line); 
	}

	app.preferences.rulerUnits = oldPref; // restore old prefs

	paperTexture.setColorSettings( oldColorSettings );
	
}

// look for any layer that starts with our "Paper Texture and delete it!"
paperTexture.runReset = function( ) {
	var targetDoc = activeDocument;
	for ( var i = 0; i < targetDoc.artLayers.length; i++ ) {
		if ( targetDoc.artLayers[i].name.search(paperTexture.tempDocName) != -1 ) {
			targetDoc.artLayers[i].remove();
			i--; // check this layer again	
		}	
	}
}

///////////////////////////////////////////////////////////////////////////////
// Function: paramsToParams
// Usage: Conver the encoded string that represents a JavaScript Object back
// into an object. All this is necessary to get non xml characters accrss the
// interface from ActionScript to JavaScript.
//        inParamsAsSource is the encoded string representing the object
//		  inVersion is 0 for 2.0.5 params and 1 for 2.0.6 params
// Return: JavaScript object for all the params from the panel
///////////////////////////////////////////////////////////////////////////////
paperTexture.paramsToParams = function( inParamsAsSource, inVersion ) {

	var params = new Object();

	try {
		// alert(inParamsAsSource);
		params = eval( decodeURIComponent( unescape( inParamsAsSource ) ) );
		for ( var i = 0; i < params.filesArray.length; i++ ) {
			// alert(params.filesArray[i]);
			// params.filesArray[i] = app.path + "/Plug-ins/Panels/Paper Texture/" + decodeURIComponent( unescape( params.filesArray[i] ) );
			params.filesArray[i] = decodeURIComponent( unescape( params.filesArray[i] ) );
			// alert(params.filesArray[i]);
		}

		// runRandomizer should be removed once we figure out best way
		if (params.runRandomizer) {
			params.filesArray = new Array();
			var rand1 = Math.floor(Math.random() * 12) + 1;
			if (rand1 > 9) {
				params.filesArray[0] = app.path + "/Plug-ins/Panels/Paper Texture/images/Texture" + rand1 + ".png";
			} else {
				params.filesArray[0] = app.path + "/Plug-ins/Panels/Paper Texture/images/Texture0" + rand1 + ".png";
			}
			var rand2 = rand1;
			while( rand1 == rand2 ) {
				rand2 = Math.floor(Math.random() * 12) + 1;
			}
			if (rand2 > 9) {
				params.filesArray[1] = app.path + "/Plug-ins/Panels/Paper Texture/images/Texture" + rand2 + ".png";
			} else {
				params.filesArray[1] = app.path + "/Plug-ins/Panels/Paper Texture/images/Texture0" + rand2 + ".png";
			}
			params.stateArray[0] = true;
			params.stateArray[1] = true;
		}
	}

	catch(e) { 
		if (paperTexture.showAlerts) alert(e +" : " + e.line); 
	}
	
	return params;
}

paperTexture.getColorSettings = function() {
	var colorSettings = {askMismatchOpening:false, askMismatchPasting:false, askMissing:false };
	try {
		var colorSettingsStr = stringIDToTypeID( "colorSettings" );
		var typeOrdinal = charIDToTypeID( "Ordn" );
		var enumTarget = charIDToTypeID( "Trgt" );
		var classProperty = charIDToTypeID( "Prpr" );
		var classApplication = charIDToTypeID( "capp" );
		var ref = new ActionReference();
		ref.putProperty( classProperty, colorSettingsStr );
		ref.putEnumerated( classApplication, typeOrdinal, enumTarget );
		var desc = executeActionGet( ref );
		var colorDesc = desc.getObjectValue( colorSettingsStr );
		var askMismatchOpeningStr = stringIDToTypeID( "askMismatchOpening" );
		var askMismatchPastingStr = stringIDToTypeID( "askMismatchPasting" );
		var askMissingStr = stringIDToTypeID( "askMissing" );
		colorSettings.askMismatchOpening = colorDesc.getBoolean( askMismatchOpeningStr );
		colorSettings.askMismatchPasting = colorDesc.getBoolean( askMismatchPastingStr );
		colorSettings.askMissing = colorDesc.getBoolean( askMissingStr );
	}
	catch(e) {
		alert(e + ":" + e.line);
	}
	return colorSettings;
}

paperTexture.setColorSettings = function( colorSettings ) {
	var idsetd = charIDToTypeID( "setd" );
    var desc3 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref2 = new ActionReference();
    var idPrpr = charIDToTypeID( "Prpr" );
    var idcolorSettings = stringIDToTypeID( "colorSettings" );
    ref2.putProperty( idPrpr, idcolorSettings );
    var idcapp = charIDToTypeID( "capp" );
    var idOrdn = charIDToTypeID( "Ordn" );
    var idTrgt = charIDToTypeID( "Trgt" );
    ref2.putEnumerated( idcapp, idOrdn, idTrgt );
    desc3.putReference( idnull, ref2 );
    var idT = charIDToTypeID( "T   " );
    var desc4 = new ActionDescriptor();
    var idaskMismatchOpening = stringIDToTypeID( "askMismatchOpening" );
    desc4.putBoolean( idaskMismatchOpening, colorSettings.askMismatchOpening );
    var idaskMismatchPasting = stringIDToTypeID( "askMismatchPasting" );
    desc4.putBoolean( idaskMismatchPasting, colorSettings.askMismatchPasting );
    var idaskMissing = stringIDToTypeID( "askMissing" );
    desc4.putBoolean( idaskMissing, colorSettings.askMissing );
    var idcolorSettings = stringIDToTypeID( "colorSettings" );
    desc3.putObject( idT, idcolorSettings, desc4 );
	executeAction( idsetd, desc3, DialogModes.NO );
}

paperTexture.getFolder = function() {
	var returnString = paperTexture.defaultFolder;
	try {
		var newF = Folder().selectDlg(localize("$$$/watermark/select/folder=Select a new folder of images"));
		if (newF != null) {
			returnString = newF.toString();
		} 
	}
	catch(e) {
		if (paperTexture.showAlerts) alert(e +" : " + e.line);
	}
	 
	return paperTexture.stringToXML( escape( encodeURIComponent( returnString ) ) );
}

paperTexture.getImages = function(folderStr)
{
	//alert(folderStr);
	var returnString = "";
	try {
		var f = new Folder(folderStr);
		var inFiles = f.getFiles("*.jpg");
		for (var i = 0; i < inFiles.length; i++) {
			var fixUp = inFiles[i].fsName;
			if ( File.fs == "Windows" ) {
				fixUp = fixUp.replace(/\\/g, "/");
			}
			returnString += unescape(fixUp);
			if (i != inFiles.length - 1) {
				returnString += "!!!";
			}
		}
		inFiles = f.getFiles("*.png");
		for (var i = 0; i < inFiles.length; i++) {
			var fixUp = inFiles[i].fsName;
			if ( File.fs == "Windows" ) {
				fixUp = fixUp.replace(/\\/g, "/");
			}
			returnString += unescape(fixUp);
			if (i != inFiles.length - 1) {
				returnString += "!!!";
			}
		}
		returnString.length = returnString.length - 1;
		var returnThis = paperTexture.stringToXML( escape( encodeURIComponent( returnString ) ) );
	}
	catch(e) {
		alert(e + ":" + e.line);
	}
	return returnThis;
}

///////////////////////////////////////////////////////////////////////////////
// Function: convertModeIfNeeded
// Usage: If the logo document is not in the correct mode things go bad,
//        check for bad modes and if so switch the mode to the target document
//        w is the current watermark file and the current target document
//        m is the new mode to convert to
// Return: watermark file converted to the target document mode if needed
///////////////////////////////////////////////////////////////////////////////
paperTexture.convertModeIfNeeded = function(w, m)
{
	if (w.mode == DocumentMode.BITMAP || 
	    w.mode == DocumentMode.INDEXEDCOLOR ||
	    w.mode == DocumentMode.MULTICHANNEL) {
		paperTexture.conditionalModeChange(m);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Function: conditionalModeChange
// Usage: convert the active document to the mode
//        m mode of the new document
// Return: activeDocument converted to the new mode via conditional mode change
///////////////////////////////////////////////////////////////////////////////
paperTexture.conditionalModeChange = function(m) {
	var eventModeChange = stringIDToTypeID( "8cba8cd6-cb66-11d1-bc43-0060b0a13dc4" );
	var descSource = new ActionDescriptor();
	var keySourceMode = charIDToTypeID( "SrcM" );
	var list = new ActionList();
	var keyCondition = charIDToTypeID( "Cndn" );
	var keyBitmap = charIDToTypeID( "UBtm" );
	var keyGrayscale = charIDToTypeID( "UGry" );
	var keyDuotone = charIDToTypeID( "UDtn" );
	var keyIndex = charIDToTypeID( "UInd" );
	var keyRGB = charIDToTypeID( "URGB" );
	var keyCMYK = charIDToTypeID( "UCMY" );
	var keyLab = charIDToTypeID( "ULab" );
	var keyMultichannel = charIDToTypeID( "UMlt" );
	list.putEnumerated( keyCondition, keyBitmap );
	list.putEnumerated( keyCondition, keyGrayscale );
	list.putEnumerated( keyCondition, keyDuotone );
	list.putEnumerated( keyCondition, keyIndex );
	list.putEnumerated( keyCondition, keyRGB );
	list.putEnumerated( keyCondition, keyCMYK );
	list.putEnumerated( keyCondition, keyLab );
	list.putEnumerated( keyCondition, keyMultichannel );
	descSource.putList( keySourceMode, list );
	var keyDestination = charIDToTypeID( "DstM" );
	var descDest = new ActionDescriptor();
	var keyMode = charIDToTypeID( "RGBM" );
	switch (m) {
		case DocumentMode.GRAYSCALE:
			keyMode = charIDToTypeID( 'Grys' );
			break;
		case DocumentMode.DUOTONE:
			keyMode = charIDToTypeID( 'DtnM' );
			break;
		case DocumentMode.CMYK:
			keyMode = charIDToTypeID( 'CMYM' );
			break;
		case DocumentMode.LAB:
			keyMode = charIDToTypeID( 'LbCM' );
			break;
		case DocumentMode.MULTICHANNEL: // keyMode = charIDToTypeID( 'MltC' );
		case DocumentMode.BITMAP: // keyMode =  = charIDToTypeID( 'BtmM' );
		case DocumentMode.INDEXEDCOLOR: //keyMode =  = charIDToTypeID( 'IndC' );
		case DocumentMode.RGB: // already done above
		default: // RGB i guess
			break;
	}
	descSource.putObject( keyDestination, keyMode, descDest );
	executeAction( eventModeChange, descSource, DialogModes.NO );
}


///////////////////////////////////////////////////////////////////////////////
// Function: duplicateLayerToDocument
// Usage: Instead of copy paste use the duplicate layer to new document command
// Input: newLayerName name for the layer in the other document
//        targetDocumentName name for the target document
// Return: <nothing> other document is altered with new layer
///////////////////////////////////////////////////////////////////////////////
paperTexture.duplicateLayerToDocument = function( newLayerName, targetDocumentName ) {
	var idDplc = charIDToTypeID( "Dplc" );
    var desc54 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref17 = new ActionReference();
        var idLyr = charIDToTypeID( "Lyr " );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref17.putEnumerated( idLyr, idOrdn, idTrgt );
    desc54.putReference( idnull, ref17 );
    var idT = charIDToTypeID( "T   " );
        var ref18 = new ActionReference();
        var idDcmn = charIDToTypeID( "Dcmn" );
        ref18.putName( idDcmn, targetDocumentName );
    desc54.putReference( idT, ref18 );
    var idNm = charIDToTypeID( "Nm  " );
    desc54.putString( idNm, newLayerName );
    var idVrsn = charIDToTypeID( "Vrsn" );
    desc54.putInteger( idVrsn, 2 );
	executeAction( idDplc, desc54, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: openDocs
// Usage: we want the current open documents that have a path if I do 
//        inputFiles = documents then I have a reference and if I add documents
//        it also adds them to my inputFiles I want to copy the current state
// Input: <none>
// Return: array of the current open docs with file references
///////////////////////////////////////////////////////////////////////////////
paperTexture.openDocs = function() {
	var docs = new Array();
	var i = 0;
	var docIndex = 0;
	var alertNonSavedDocs = false;
	var nonSavedDocs = new Array();
	for ( var i = 0; i < app.documents.length; i++ ) {
		try {
			var temp = app.documents[ i ].fullName;
			docs[ docIndex ] = app.documents[ i ];
			docIndex++;
		}
		catch( e ) {
			if ( e.number == 8103 ) { // this document has not been saved
				alertNonSavedDocs = true;
				nonSavedDocs.push( app.documents[ i ].name );
			} else {
				throw e;
			}
		}
	}
	if ( alertNonSavedDocs ) {
		alert( localize("$$$/watermark/alert/openfileneedsave=Open files must be saved before they can be used by the Watermark Creator.^r The following files will not be processed.^r( {0} )").replace("{0}", nonSavedDocs) );
	}
	return docs;
}

///////////////////////////////////////////////////////////////////////////////
// Function: IsFileOneOfThese
// Usage: Given a file see if the extension is in the list
// Input: inFileName a file with an extension
//        inArrayOfFileExtensions an array of extensions
// Return: true if the file extension is in the list
///////////////////////////////////////////////////////////////////////////////
paperTexture.isFileOneOfThese = function( inFileName, inArrayOfFileExtensions ) {
	var lastDot = inFileName.toString().lastIndexOf( "." );
	if ( lastDot == -1 ) {
		return false;
	}
	var strLength = inFileName.toString().length;
	var extension = inFileName.toString().substr( lastDot + 1, strLength - lastDot );
	extension = extension.toUpperCase();
	for (var i = 0; i < inArrayOfFileExtensions.length; i++ ) {
		if ( extension == inArrayOfFileExtensions[i] ) {
			return true;
		}
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////
// Function: isFileOneOfThese
// Usage: Given a file name and a list of types, only runs on Mac
// Dependant: isMacintoshOS
// Input: inFileName a full path to a file
//        inArrayOfFileTypes an array of types
// Return: true if the file type is in the list
///////////////////////////////////////////////////////////////////////////////
paperTexture.isFileOneOfTheseTypes = function( inFileName, inArrayOfFileTypes ) {
	if ( ! paperTexture.isMacintoshOS() ) {
		return false;
		}
	var file = new File (inFileName);
	for (var i = 0; i < inArrayOfFileTypes.length; i++ ) {
		if ( file.type == inArrayOfFileTypes[i] ) {
			return true;
		}
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////
// Function: IsMacintoshOS
// Usage: Are we running on the Macintosh OS?
// Input: <none>
// Return: true if on a macintosh
///////////////////////////////////////////////////////////////////////////////
paperTexture.isMacintoshOS = function() {
	if ( $.os.search(/macintosh/i) != -1 ) {
		return true;
	} else {
		return false;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Function: stringToUnitValue
// Usage: Convert a string to a UnitValue, "6px" "6 px" et. al.
// Input: inString of the number and units, could just be number
//        inUnits if no units detected or bad units this will be the default
// Return: UnitValue
///////////////////////////////////////////////////////////////////////////////
paperTexture.stringToUnitValue = function(inString, inUnits) {
	var unitValue = "";
	var unitType = "";
	var findingNumbers = 0; // which includes '+' or '-' characters
	var findingStrings = 1;
	var findingWhat = findingNumbers;
	
	for (var i = 0; i < inString.length; i++) {
		if ( ( findingWhat == findingNumbers ) && 
		     ( ( inString[i] >= "0" && inString[i] <= "9" ) ||
		     ( inString[i] == "." ) || 
		     ( inString[i] == '-' ) ||
		     ( inString[i] == '+' ) ) ) {
			unitValue += inString[i];
		} else {
			findingWhat = findingStrings;
		}
		if (findingWhat == findingStrings && inString[i] != " ") {
			unitType += inString[i];
		}
	}

	// check for a valid unit types, these are from the documentation
	// "centimeters" and "centimeter" is actually valid but come on,
	// nobody would type those into a edit text box
	if (unitType.length == 0 || ( unitType != "px"  && unitType != "in"  &&
	        unitType != "ft" && unitType != "yd"  && unitType != "mi"  && 
	        unitType != "mm" && unitType != "cm"  && unitType != "m"   && 
	        unitType != "km" && unitType != "pt"  && unitType != "pc"  &&
	        unitType != "tpt"&& unitType != "tpc" && unitType != "ci" &&
	        unitType != "%" ) ) {

	    unitType = inUnits;
	}

	return new UnitValue(unitValue, unitType);
}

///////////////////////////////////////////////////////////////////////////////
// Function: fitOnScreen
// Usage: Fit the current image on screen for best view
// Input: <nothing> an open document
// Return: <nothing> altered view
///////////////////////////////////////////////////////////////////////////////
paperTexture.fitOnScreen = function() {
	var idslct = charIDToTypeID( "slct" );
    var desc106 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref63 = new ActionReference();
    var idMn = charIDToTypeID( "Mn  " );
    var idMnIt = charIDToTypeID( "MnIt" );
    var idFtOn = charIDToTypeID( "FtOn" );
    ref63.putEnumerated( idMn, idMnIt, idFtOn );
    desc106.putReference( idnull, ref63 );
	executeAction( idslct, desc106, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// The following routines are what transfer data in and out of the swf panel
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// Function: stringToXML
// Usage: Returns to the panel must be wrapped in XML syntax
// Input: inString string to wrap in XML
// Return: string in XML syntax
///////////////////////////////////////////////////////////////////////////////
paperTexture.stringToXML = function(inString) {
    var s = '<object>';
    s += paperTexture.propertyToXML(0, "error");
    s += paperTexture.propertyToXML(inString, "message");
    s += '</object>';
    return s;
}

///////////////////////////////////////////////////////////////////////////////
// Function: errorToXML
// Usage: Returns to the panel must be wrapped in XML syntax
// Input: inError is an Error object
// Return: Error in XML syntax
///////////////////////////////////////////////////////////////////////////////
paperTexture.errorToXML = function(inError) {
    var s = '<object>';
    s += paperTexture.propertyToXML(inError.number, "error");
    s += paperTexture.propertyToXML(inError.message, "message");
    s += paperTexture.propertyToXML(inError.line, "line");
    s += paperTexture.propertyToXML(inError.fileName, "fileName");
    s += '</object>';
    return s;
}

///////////////////////////////////////////////////////////////////////////////
// Function: propertyToXML
// Usage: Returns to the panel must be wrapped in XML syntax, helper routine
// Input: inProperty property string
//        inID property id
// Return: property wrapped in XML syntax for return value to the panel
///////////////////////////////////////////////////////////////////////////////
paperTexture.propertyToXML = function(inProperty, inID) {
    var t = typeof inProperty;
    var s = '<property id="' + inID + '">';
    switch (t) {
        case "number":
            s += '<number>' + inProperty.toString() + '</number>';
            break;
        case "boolean":
            s += '<' + inProperty.toString() + '/>';
            break;
        case "string":
            s += '<string>' + inProperty.toString() + '</string>';
            break;
        // TODO figure this out
        case "object":
            //s += '<object>';
			//for (var i in inProperty) {
			//	s += propertyToXML(inProperty[i], i.toString());
			//}
			//s += '</object>';
            break;
        case "undefined":
            s += '<string>undefined</string>';
            break;
        default:
            if (paperTexture.debugAlerts) alert('unknown type not supported: ' + t);
            break;
    }
    s += '</property>';
    return s;
}

///////////////////////////////////////////////////////////////////////////////
// Function: getZstring
// Usage: Called from the panel so we can get localized zstring
// Input: zstring string
// Return: XML describing the full path to the folder
///////////////////////////////////////////////////////////////////////////////
paperTexture.getZstring = function(zstring) {
	var s = '<object>';
	s += paperTexture.propertyToXML(escape(encodeURIComponent(localize(decodeURIComponent(unescape(zstring))))), "zstring");
	s += '</object>';
	return s;
}

paperTexture.myAlert = function(zstring) {
	alert(localize(decodeURIComponent(unescape(zstring))));
}

// version 0 is the 2.0.5 way where I just evaled the file
// and it turned into an object
// version 1 is the 2.0.6 way where I do the 2.0.5 way
// then i decode and unescape each piece
// I'll write out a marker to these files and hopefully
// be able to read both formats back
paperTexture.saveParamsToDisk = function( inParamsAsSource ) {
	try {
	var saveFile = null;
	Folder.current = Folder( "~/" );
	if ( File.fs == "Windows" ) {
		saveFile = File.saveDialog( localize( "$$$/JavaScripts/PaperTexture/PickSave=Pick a Paper Texture file to save" ),
		                            localize( "$$$/JavaScripts/PaperTexture/PickWin=Paper Texture Files: *.pap" ) );
	} else {
		saveFile = File.saveDialog( localize( "$$$/JavaScripts/PaperTexture/PickSave=Pick a Paper Texture file to save" ),
		                            paperTexture.macFilter );
	}
	if ( saveFile != null ) {
		if ( saveFile.exists && File.fs == "Windows" ) { // mac does this warning for us
			if ( confirm(localize( "$$$/JavaScripts/ImageProcessor/FileAlreadyExists=The file already exists. Do you want to replace?" )) == false) {
				return;
			}
		}
		saveFile.open( "w", "TEXT", "????" );
		saveFile.write( "1, " + inParamsAsSource );
		saveFile.close();
	}
	}
	catch(e) {
		var s = paperTexture.errorToXML(e);
		alert(s);
	}
}

// see saveParamsToDisk comments
paperTexture.loadParamsFromDisk = function() {
	var s = "";
	try {
		// var params = new Object();
		var loadFile = null;
		Folder.current = Folder( "~/" );
		if ( File.fs == "Windows" ) {
			loadFile = File.openDialog( localize( "$$$/JavaScripts/PaperTexture/PickLoad=Pick a Paper Texture file to load" ), 
			                            localize( "$$$/JavaScripts/PaperTexture/PickWin=Paper Texture Files: *.pap" ) );
		} else {
			loadFile = File.openDialog( localize( "$$$/JavaScripts/PaperTexture/PickLoad=Pick a Paper Texture file to load" ),
			                            paperTexture.macFilter );
		}
		if ( loadFile != null && loadFile.exists ) {
			loadFile.open( "r" );
			s = '<object>';
    		s += paperTexture.propertyToXML(0, "error");
    		var inDecoded = loadFile.read(loadFile.length);
			loadFile.close();
    		var version = inDecoded.split(",");
    		// 2.0.5 wrote out with no comma
    		// 2.0.6 unoffical release is not supported
    		if (version.length == 1) {
    			version = 0;
    		} else if (version.length == 2 && (Number(version[0]) == 1)) {
    			inDecoded = version[1];
    			version = 1;
    		} else {
    			s = '<object>';
   				s += paperTexture.propertyToXML(3, "error");
				s += '</object>';
				if (paperTexture.debugAlerts) {
					alert(s);
				}
				return s;
    		}
    		var inObject = paperTexture.paramsToParams(inDecoded, version);
    		s += paperTexture.propertyToXML(inObject.rbUseLogoFile, "rbUseLogoFile"); 
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.etWatermarkText)), "etWatermarkText");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.ddFontFamily)), "ddFontFamily");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.ddFontStyle)), "ddFontStyle");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.etTextSize)), "etTextSize"); 
   			s += paperTexture.propertyToXML(inObject.cpTextColorPicker, "cpTextColorPicker");
   			s += paperTexture.propertyToXML(inObject.cbAppendFileNames, "cbAppendFileNames");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.stSelectedWatermark)), "stSelectedWatermark");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.stSelectedInput)), "stSelectedInput");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.stSelectedOutput)), "stSelectedOutput");
   			s += paperTexture.propertyToXML(inObject.cbUseOpenImages, "cbUseOpenImages");
   			s += paperTexture.propertyToXML(inObject.ddPosition, "ddPosition");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.etWaterMarkSize)), "etWaterMarkSize");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.etWaterMarkOpacity)), "etWaterMarkOpacity");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.etVerticalOffset)), "etVerticalOffset");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.etHorizontalOffset)), "etHorizontalOffset");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.etQuality)), "etQuality");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.etResolution)), "etResolution");
   			s += paperTexture.propertyToXML(inObject.cbSizeToFit, "cbSizeToFit");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.etSizeToFitW)), "etSizeToFitW");
   			s += paperTexture.propertyToXML(escape(encodeURIComponent(inObject.etSizeToFitH)), "etSizeToFitH");
   			s += paperTexture.propertyToXML(inObject.rbSaveForWeb, "rbSaveForWeb");
			s += '</object>';
		} else { // this isn't even a file or user canceled
			s = '<object>';
   			s += paperTexture.propertyToXML(2, "error");
			s += '</object>';
			if (paperTexture.debugAlerts) {
				alert(s);
			}
		}
	} 
	catch(e) {
		s = paperTexture.errorToXML(e);
		if (paperTexture.debugAlerts) {
			alert(s);
		}
	}
	return s;
}

///////////////////////////////////////////////////////////////////////////////
// Function: ReadHeader
// Usage: a very crude xml parser, this reads the "Tag" of the <Tag>Data</Tag>
// Input:
// Return:
///////////////////////////////////////////////////////////////////////////////
paperTexture.readHeader = function( inFile ) {
	var returnValue = "";
	if ( ! inFile.eof ) {
		var c = "";
		while ( c != "<" && ! inFile.eof ) {
			c = inFile.read( 1 );
		}
		while ( c != ">" && ! inFile.eof ) {
			c = inFile.read( 1 );
			if ( c != ">" ) {
				returnValue += c;
			}
		}
	} else {
		returnValue = "end of file";
	}
	return returnValue;
}

///////////////////////////////////////////////////////////////////////////////
// Function: ReadData
// Usage: very crude xml parser, this reads the "Data" of the <Tag>Data</Tag>
// Input:
// Return:
///////////////////////////////////////////////////////////////////////////////
paperTexture.readData = function( inFile ) {
	var returnValue = "";
	if ( ! inFile.eof ) {
		var c = "";
		while ( c != "<" && ! inFile.eof ) {
			c = inFile.read( 1 );
			if ( c != "<" ) {
				returnValue += c;
			}
		}
		inFile.seek( -1, 1 );
	}
	return returnValue;
}

/////////////////////////////////////////////////////////////////////
// Function: MacXMLFilter
// Input: f, file or folder to check
// Return: true or false, true if file or folder is to be displayed
/////////////////////////////////////////////////////////////////////
paperTexture.macFilter = function( f ) {
	var wtmExtension = ".wtm";
	var lCaseName = f.name;
	lCaseName.toLowerCase();
	if ( lCaseName.indexOf( wtmExtension ) == f.name.length - wtmExtension.length )
		return true;
	else if ( f.type == 'TEXT' )
		return true;
	else if ( f instanceof Folder )
		return true;
	else
		return false;
}

/////////////////////////////////////////////////////////////////////
// Function: Help
// Input: <none>
// Return: <none> but hopefully a web browser got launched
/////////////////////////////////////////////////////////////////////
paperTexture.help = function( ) {
	var s = "";
	try {
		
		s = "<HTML><HEAD><META HTTP-EQUIV=REFRESH CONTENT=";
		s += '"0; URL=';
		s += localize("$$$/papertexture/url/helpmovie=http://flypapertextures.com/texture-packs-2/");
		s += ' ">';
		s += "</HEAD><BODY>";
		s += "<!---CENTER><BR><BR><b><i><span style='font-family:Arial'>";
		s += localize("$$$/watermark/str/redirect=redirecting to documentation...");
		s += "</span></i></b></center--->";
		s += "</BODY></HTML>";
		
		var helpFile = new File( Folder.temp.toString() + '/paperTexture.html' );
		if (helpFile.open('w')) {
			helpFile.write( s );
			helpFile.close();
			helpFile.execute();
			s = paperTexture.stringToXML("Success");
		}
	}
	catch(e) { 
		if (paperTexture.showAlerts) alert(e + " : " + e.line); 
	}
	return s;
}


///////////////////////////////////////////////////////////////////////////////
// Function: ConvertToProfile
// Usage: convert to given profile
// Input: profile name as string
// Return: <none> active document converted to profile
///////////////////////////////////////////////////////////////////////////////
paperTexture.convertToProfile = function( inProfile ) {
	if ( 'Untagged RGB' == inProfile || 'Same as Source' == inProfile ) {
		return;
	}
	app.activeDocument.convertProfile( inProfile, Intent.RELATIVECOLORIMETRIC, true, true );
}

paperTexture.addMaskAsNeeded = function ( reveals ) {
	var idMk = charIDToTypeID( "Mk  " );
	var desc13 = new ActionDescriptor();
	var idNw = charIDToTypeID( "Nw  " );
	var idChnl = charIDToTypeID( "Chnl" );
	desc13.putClass( idNw, idChnl );
	var idAt = charIDToTypeID( "At  " );
	var ref9 = new ActionReference();
	var idChnl = charIDToTypeID( "Chnl" );
	var idChnl = charIDToTypeID( "Chnl" );
	var idMsk = charIDToTypeID( "Msk " );
	ref9.putEnumerated( idChnl, idChnl, idMsk );
	desc13.putReference( idAt, ref9 );
	var idUsng = charIDToTypeID( "Usng" );
	var idUsrM = charIDToTypeID( "UsrM" );
	var idRvlA = charIDToTypeID( reveals ? "RvlA" : "HdAl" );
	desc13.putEnumerated( idUsng, idUsrM, idRvlA );
	executeAction( idMk, desc13, DialogModes.NO );
}

paperTexture.textToBlendMode = function( blendModeText ) {
	switch ( blendModeText ) {
    	case localize('$$$/TransferMode/Dissolve=Dissolve'):            	return BlendMode.DISSOLVE;
		case localize('$$$/TransferMode/Darken=Darken'):              		return BlendMode.DARKEN;
		case localize('$$$/TransferMode/Multiply=Multiply'):            	return BlendMode.MULTIPLY;
		case localize('$$$/TransferMode/ColorBurn=Color Burn'):          	return BlendMode.COLORBURN;
		case localize('$$$/TransferMode/LinearBurn=Linear Burn'):         	return BlendMode.LINEARBURN;
		case localize('$$$/TransferMode/DarkerColor=Darker Color'):        	return BlendMode.DARKERCOLOR;
		case localize('$$$/TransferMode/Lighten=Lighten'):             		return BlendMode.LIGHTEN;
		case localize('$$$/TransferMode/Screen=Screen'):              		return BlendMode.SCREEN;
		case localize('$$$/TransferMode/ColorDodge=Color Dodge'):         	return BlendMode.COLORDODGE;
		case localize('$$$/TransferMode/LinearDodge=Linear Dodge (Add)'):  	return BlendMode.LINEARDODGE
		case localize('$$$/TransferMode/LighterColor=Lighter Color'):       return BlendMode.LIGHTERCOLOR;
		case localize('$$$/TransferMode/Overlay=Overlay'):             		return BlendMode.OVERLAY;
		case localize('$$$/TransferMode/SoftLight=Soft Light'):          	return BlendMode.SOFTLIGHT;
		case localize('$$$/TransferMode/HardLight=Hard Light'):          	return BlendMode.HARDLIGHT;
		case localize('$$$/TransferMode/VividLight=Vivid Light'):         	return BlendMode.VIVIDLIGHT;
		case localize('$$$/TransferMode/LinearLight=Linear Light'):        	return BlendMode.LINEARLIGHT;
		case localize('$$$/TransferMode/PinLight=Pin Light'):           	return BlendMode.PINLIGHT;
		case localize('$$$/TransferMode/HardMix=Hard Mix'):            		return BlendMode.HARDMIX;
		case localize('$$$/TransferMode/Difference=Difference'):          	return BlendMode.DIFFERENCE;
		case localize('$$$/TransferMode/Exclusion=Exclusion'):           	return BlendMode.EXCLUSION;
		case localize('$$$/TransferMode/Subtract=Subtract'):            	return BlendMode.SUBTRACT;
		case localize('$$$/TransferMode/Divide=Divide'):              		return BlendMode.DIVIDE;
		case localize('$$$/TransferMode/Hue=Hue'):                 			return BlendMode.HUE;
		case localize('$$$/TransferMode/Saturation=Saturation'):          	return BlendMode.SATURATION;
		case localize('$$$/TransferMode/Color=Color'):               		return BlendMode.COLORBLEND;
		case localize('$$$/TransferMode/Luminosity=Luminosity'):          	return BlendMode.LUMINOSITY;
	}
	return BlendMode.NORMAL;
}

// i didn't indent cuz that would be annoying
} // end overall try catch block
catch(e) {
	if (paperTexture.showAlerts) {
		alert(localize("$$$/watermark/alert/alert_key_value={0} : {1}").replace("{0}", e).replace("{1}", e.line));
	}
}

// end PaperTexture.jsx
